package pps.core.prediction.service;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import org.apache.commons.lang3.StringUtils;
import pps.core.auth.HttpRequestClient;
import pps.core.common.cache.ThirdPartyConfigCache;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.service.data.wellhead_daily_production_situation.QueryWellheadDailyProductionSituationInput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.QueryWellheadDailyProductionSituationOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XHttpUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 井口生产情况模块(测试用)
 * todo: 模拟测试用,后续替换
 *
 * @author ZWT
 * @date 2023/09/27
 */
@XText("井口生产情况模块")
@XService
public class WellheadDailyProductionSituationService {


    /**
     * 井口生产情况--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryWellheadDailyProductionSituationOutput}>
     */
    @XText("井口生产情况--分页列表")
    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryWellheadDailyProductionSituationOutput> queryWellheadDailyProductionSituationView(XContext context, QueryWellheadDailyProductionSituationInput input) {
        WellheadDailyProductionSituationMapper mapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        LambdaQueryWrapper<WellheadDailyProductionSituationEnt> queryWrapper = new LambdaQueryWrapper<WellheadDailyProductionSituationEnt>()
                .eq(WellheadDailyProductionSituationEnt::getWellheadId, input.getWellheadId())
                .eq(WellheadDailyProductionSituationEnt::getWellNumber, input.getWellNumber())
                .orderByDesc(WellheadDailyProductionSituationEnt::getDataDate);
        String executionCycle = input.getExecutionCycle();
        if (StringUtils.isNotEmpty(executionCycle)) {
            String[] split = StringUtils.split(executionCycle, "-");
            queryWrapper.ge(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[0], "yyyy年MM月dd日"))
                    .le(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[1], "yyyy年MM月dd日"));
        }
        List<WellheadDailyProductionSituationEnt> list = mapper.selectList(queryWrapper);
        PageInfo<WellheadDailyProductionSituationEnt> pageInfo = new PageInfo<>(list);
        List<QueryWellheadDailyProductionSituationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryWellheadDailyProductionSituationOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XText("井口生产情况--test")
    @XApiGet
    public XServiceResult test(XContext context) {
        context.getLogger().info("---------------------------cqToken---------------------------");
        String cqToken = "";
        String api = "";
        try {
            cqToken = this.getToken(context);
            context.getLogger().info("cqToken: " + cqToken);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        Map<String, String> header = new HashMap<>(4);
        header.put("token", cqToken);
        context.getLogger().info("---------------------------CQ_WELL_TECH_DAILY---------------------------");
        HttpRequestClient client = new HttpRequestClient(cqToken);
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_TECH_DAILY;
            Map<String, Object> param = new HashMap<>(4);
            param.put("Calc_Date", "2024-03-15");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_WELL_TECH_DAILY: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_WELL_STATION_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_WELL_STATION_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_RPT_SYSTEM_START_STOP---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_RPT_SYSTEM_START_STOP;
            Map<String, Object> param = new HashMap<>(4);
            param.put("Org_Name", "第一采油厂");
            param.put("WELL_NAME", "丹161");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_RPT_SYSTEM_START_STOP: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_WELL_REAL_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_WELL_REAL_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XServiceResult.OK;
    }

    /*----------------------------------- 私有方法 -----------------------------------*/

    /**
     * 获取长庆TOKEN
     *
     * @param context 上下文
     * @return {@link String}
     */
    private String getToken(XContext context) {
        String token;
        ThirdPartyConfigCache exist = ThirdPartyConfigCache.exist(context, ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        if (ObjectUtil.isNull(exist)) {
            token = this.getCqToken(context);
        } else {
            long duration = (long) exist.getDuration();
            if (duration - DateUtil.between(exist.getCurrentDate(), DateUtil.date(), DateUnit.SECOND) < ThirdPartyApiConstant.MAX_SECOND) {
                token = this.getCqToken(context);
            } else {
                token = exist.getCode();
            }
        }
        return token;
    }

    /**
     * 获取长庆Token
     *
     * @return {@link String}
     */
    private String getCqToken(XContext context) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_TOKEN;
        Map<String, String> param = new HashMap<>(4);
        param.put("UserAccount", "PvGroup");
        param.put("UserPassWord", "PvGroup@147369");
        String result = XHttpUtils.postAsForm(url, param);
        JSONObject jsonObject = JSONUtil.parseObj(result);
        String token = jsonObject.getStr("token");
        this.setThirdPartyConfigCache(context, token);
        return token;
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param token   代币
     */
    private void setThirdPartyConfigCache(XContext context, String token) {
        ThirdPartyConfigCache cache = new ThirdPartyConfigCache();
        cache.setCodeKey(ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        cache.setCode(token);
        cache.setValidity(90);
        cache.setCurrentDate(DateUtil.date());
        ThirdPartyConfigCache.set(context, cache);
    }
}