package pps.core.prediction.utils;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import pps.core.common.cache.ThirdPartyConfigCache;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XHttpUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2024/03/18 15:07
 */
public class ServiceUtil {

    /**
     * 获取长庆TOKEN
     *
     * @param context 上下文
     * @return {@link String}
     */
    public static String getToken(XContext context) {
        String token;
        ThirdPartyConfigCache exist = ThirdPartyConfigCache.exist(context, ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        if (ObjectUtil.isNull(exist)) {
            token = ServiceUtil.getCqToken(context);
        } else {
            long duration = (long) exist.getDuration();
            if (duration - DateUtil.between(exist.getCurrentDate(), DateUtil.date(), DateUnit.SECOND) < ThirdPartyApiConstant.MAX_SECOND) {
                token = ServiceUtil.getCqToken(context);
            } else {
                token = exist.getCode();
            }
        }
        return token;
    }

    /*----------------------------------- 私有方法 -----------------------------------*/

    /**
     * 获取长庆Token
     *
     * @return {@link String}
     */
    private static String getCqToken(XContext context) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_TOKEN;
        Map<String, String> param = new HashMap<>(4);
        param.put("UserAccount", "PvGroup");
        param.put("UserPassWord", "PvGroup@147369");
        String result = XHttpUtils.postAsForm(url, param);
        JSONObject jsonObject = JSONUtil.parseObj(result);
        String token = jsonObject.getStr("token");
        ServiceUtil.setThirdPartyConfigCache(context, token);
        return token;
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param token   代币
     */
    private static void setThirdPartyConfigCache(XContext context, String token) {
        ThirdPartyConfigCache cache = new ThirdPartyConfigCache();
        cache.setCodeKey(ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        cache.setCode(token);
        cache.setValidity(90);
        cache.setCurrentDate(DateUtil.date());
        ThirdPartyConfigCache.set(context, cache);
    }
}
